#[non_exhaustive]pub enum HostError {
PipeError(PipeError),
RequestFailed,
InvalidDescriptor,
OutOfSlots,
OutOfPipes,
NoSuchDevice,
InsufficientMemory,
Other(&'static str),
}Expand description
Indicates type of error of Host interface
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
PipeError(PipeError)
A pipe-level transfer error occurred.
RequestFailed
The control request was not acknowledged by the device.
InvalidDescriptor
A descriptor returned by the device could not be parsed.
OutOfSlots
No free device slots available.
OutOfPipes
No free host pipes available.
NoSuchDevice
The addressed device does not exist.
InsufficientMemory
Insufficient memory for the requested operation.
Other(&'static str)
An unspecified error with a static description.