embassy-usb-driver

Crates

git

Versions

default

Flavors

TimeoutConfig

Struct TimeoutConfig 

#[non_exhaustive]
pub struct TimeoutConfig { pub data_timeout: Duration, pub no_data_timeout: Duration, }
Timeouts applied to a control pipe’s NAK-retry behaviour.

Fields (Non-exhaustive)

This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.
data_timeout: Duration

Maximum response timeout for transactions with a Data Stage.

no_data_timeout: Duration

Maximum response timeout for transactions without a Data Stage.